"""Helper functions."""

import logging
from datetime import datetime, timezone
from string import digits
from typing import Optional

HS_UNIT_A = "A"
HS_UNIT_AMPERES = "Amperes"
HS_UNIT_CELSIUS = "C"
HS_UNIT_FAHRENHEIT = "F"
HS_UNIT_KW = "kW"
HS_UNIT_KWH = "kW Hours"
HS_UNIT_LUX = "Lux"
HS_UNIT_PERCENTAGE = "%"
HS_UNIT_V = "V"
HS_UNIT_VOLTS = "Volts"
HS_UNIT_W = "W"
HS_UNIT_WATTS = "Watts"

HS_NULL_DATE = "-62135596800000"

_LOGGER = logging.getLogger(__name__)


def get_datetime_from_last_change(last_change: str) -> Optional[datetime]:
    """Parses a last_change to return a python datetime object, or None if no datetime can be parsed."""
    if HS_NULL_DATE in last_change:
        return None

    if "-" in last_change:
        lc = last_change.split("-")[0]
    else:
        lc = last_change

    date_string = "".join(i for i in lc if i in digits)

    try:
        unix_timestamp = int(date_string) / 1000
        dt = datetime.fromtimestamp(unix_timestamp, tz=timezone.utc)
    except ValueError:
        return None

    return dt


def get_uom_from_status(status: str) -> Optional[str]:
    """Parses a status to return a unit of measure, or None if no unit can be parsed."""
    uom = None
    if HS_UNIT_AMPERES in status:
        uom = HS_UNIT_AMPERES
    elif HS_UNIT_A in status:
        uom = HS_UNIT_A
    elif HS_UNIT_CELSIUS in status:
        uom = HS_UNIT_CELSIUS
    elif HS_UNIT_FAHRENHEIT in status:
        uom = HS_UNIT_FAHRENHEIT
    elif HS_UNIT_KWH in status:
        uom = HS_UNIT_KWH
    elif HS_UNIT_KW in status:
        uom = HS_UNIT_KW
    elif HS_UNIT_LUX in status:
        uom = HS_UNIT_LUX
    elif HS_UNIT_PERCENTAGE in status:
        uom = HS_UNIT_PERCENTAGE
    elif HS_UNIT_VOLTS in status:
        uom = HS_UNIT_VOLTS
    elif HS_UNIT_V in status:
        uom = HS_UNIT_V
    elif HS_UNIT_WATTS in status:
        uom = HS_UNIT_WATTS
    elif HS_UNIT_W in status:
        uom = HS_UNIT_W
    return uom



"""
Models a connection to a HomeSeer software installation.
Sends commands via JSON API and listens for device changes via ASCII interface.
"""

from aiohttp import BasicAuth, ClientSession, ContentTypeError
from asyncio import TimeoutError
import logging

from .const import (
    DEFAULT_ASCII_PORT,
    DEFAULT_HTTP_PORT,
    DEFAULT_PASSWORD,
    DEFAULT_USERNAME,
)
from .devices import get_device
from .events import HomeSeerEvent
from .listener import Listener

_LOGGER = logging.getLogger(__name__)


class HomeSeer:
    """Representation of a HomeSeer software installation."""

    def __init__(
        self,
        host: str,
        websession: ClientSession,
        username: str = DEFAULT_USERNAME,
        password: str = DEFAULT_PASSWORD,
        http_port: int = DEFAULT_HTTP_PORT,
        ascii_port: int = DEFAULT_ASCII_PORT,
    ) -> None:
        self._host = host
        self._websession = websession
        self._auth = BasicAuth(username, password)
        self._http_port = http_port
        self._listener = Listener(
            self._host,
            username=username,
            password=password,
            ascii_port=ascii_port,
            async_message_callback=self._message_callback,
            async_connect_callback=self._connect_callback,
            async_disconnect_callback=self._disconnect_callback,
        )
        self._available = False
        self._devices = {}
        self._events = []

    @property
    def available(self) -> bool:
        """Return True if self._listener is connected to the ASCII connection."""
        return self._available

    @property
    def devices(self) -> dict:
        """Return a dict of initialized supported devices indexed by device_ref for the HomeSeer instance."""
        return self._devices

    @property
    def events(self) -> list:
        """Return a list of initialized events for the HomeSeer instance."""
        return self._events

    async def initialize(self) -> None:
        """"Retrieve devices and events from the HomeSeer instance."""
        await self._get_devices()
        await self._get_events()

    async def start_listener(self) -> None:
        """Start the ASCII listener to listen for device changes."""
        await self._listener.start()

    async def stop_listener(self) -> None:
        """Stop the ASCII listener."""
        await self._listener.stop()

    async def control_device_by_value(self, ref: int, value: int) -> None:
        """
        Provides an interface for controlling a device by value
        directly through the HomeSeer object.
        """
        params = {
            "request": "controldevicebyvalue",
            "ref": ref,
            "value": value,
        }
        await self._request("get", params=params)

    async def _request(self, method, params=None, json=None) -> dict:
        """Make a request to the HomeSeer JSON API."""
        url = f"http://{self._host}:{self._http_port}/JSON"

        try:
            async with self._websession.request(
                method,
                url,
                params=params,
                json=json,
                auth=self._auth,
            ) as result:
                result.raise_for_status()
                _LOGGER.debug(
                    f"HomeSeer request response from {self._host}: {await result.text()}"
                )
                return await result.json()

        except ContentTypeError:
            _LOGGER.debug(
                f"HomeSeer returned non-JSON response from {self._host}: {await result.text()}"
            )

        except TimeoutError:
            _LOGGER.error(f"Timeout while requesting HomeSeer data from {self._host}")

        except Exception as ex:
            _LOGGER.error(f"HomeSeer HTTP Request error from {self._host}: {ex}")

    async def _get_devices(self) -> None:
        """Populate supported devices from HomeSeer API."""
        _LOGGER.debug(f"Requesting HomeSeer devices from {self._host}")
        try:
            params = {"request": "getstatus"}
            result = await self._request("get", params=params)

            all_devices = result["Devices"]

            params = {"request": "getcontrol"}
            result = await self._request("get", params=params)

            control_data = result["Devices"]

            for device in all_devices:
                dev = get_device(device, control_data, self._request)
                if dev is not None:
                    self._devices[dev.ref] = dev

        except TypeError:
            _LOGGER.error(f"Error retrieving HomeSeer devices from {self._host}")

    async def _get_events(self) -> None:
        """Populate supported events from HomeSeer API."""
        _LOGGER.debug(f"Requesting HomeSeer events from {self._host}")
        try:
            params = {"request": "getevents"}
            result = await self._request("get", params=params)

            all_events = result["Events"]

            for event in all_events:
                ev = HomeSeerEvent(event, self._request)
                self._events.append(ev)

        except TypeError:
            _LOGGER.error(f"Error retrieving HomeSeer events from {self._host}")

    async def _message_callback(self, device_ref: str) -> None:
        """Called by the ASCII listener when a Device Change message is received."""
        try:
            device = self.devices[int(device_ref)]
        except KeyError:
            _LOGGER.debug(
                f"Device Change message received for unsupported or uninitialized device "
                f"from {self._host}: device ref {device_ref}"
            )
            return

        params = {"request": "getstatus", "ref": device.ref}
        _LOGGER.debug(f"Requesting updated data for device ref {device_ref}")
        try:
            result = await self._request("get", params=params)
            for raw_dev in result["Devices"]:
                if int(raw_dev["ref"]) == device.ref:
                    device.update_data(raw_dev)
        except Exception as ex:
            _LOGGER.error(
                f"Error retrieving updated data for device ref {device.ref} from {self._host}: {ex}"
            )

    async def _connect_callback(self) -> None:
        """Called by the ASCII listener after an ASCII connection is established."""
        _LOGGER.debug(
            f"Refreshing devices for {self._host} and setting availability to True"
        )
        self._available = True

        try:
            params = {"request": "getstatus"}
            result = await self._request("get", params=params)
            homeseer_devices = result["Devices"]

        except TypeError:
            _LOGGER.error(f"Error refreshing HomeSeer data from {self._host}")
            return

        for raw_device in homeseer_devices:
            try:
                device = self.devices[int(raw_device["ref"])]
                device.update_data(new_data=raw_device, connection_flag=True)
            except KeyError:
                _LOGGER.debug(
                    f"HomeSeer refresh data retrieved for unsupported or uninitialized device from {self._host}: "
                    f"device ref {raw_device['ref']} ({raw_device})"
                )

    async def _disconnect_callback(self) -> None:
        """Called by the ASCII listener after an ASCII connection is disconnected."""
        _LOGGER.debug(f"Setting availability for {self._host} to False")
        self._available = False

        for device in self.devices.values():
            device.update_data(connection_flag=True)



"""ASCII (Telnet) listener for HomeSeer."""

import asyncio
import logging

from .const import DEFAULT_ASCII_PORT, DEFAULT_USERNAME, DEFAULT_PASSWORD
from .errors import HomeSeerASCIIConnectionError

PING_TIMER = 60
RECONNECT_TIMER = 10
STATE_CONNECTED = "connected"
STATE_IDLE = "idle"
STATE_STOPPED = "stopped"

_LOGGER = logging.getLogger(__name__)


class Listener:
    def __init__(self, host, **kwargs):
        self._host = host
        self._port = kwargs.get("ascii_port", DEFAULT_ASCII_PORT)
        self._username = kwargs.get("username", DEFAULT_USERNAME)
        self._password = kwargs.get("password", DEFAULT_PASSWORD)
        self._async_message_callback = kwargs.get("async_message_callback")
        self._async_connect_callback = kwargs.get("async_connect_callback")
        self._async_disconnect_callback = kwargs.get("async_disconnect_callback")
        self._reader = None
        self._writer = None
        self._state = STATE_IDLE
        self._ping_task = None
        self._ping_flag = False

    @property
    def state(self):
        return self._state

    async def start(self):
        """Start the ASCII listener."""
        self._state = STATE_IDLE
        if await self._open_connection():
            asyncio.get_event_loop().create_task(self._listen())
            self._ping_task = asyncio.get_event_loop().create_task(self._ping())
        else:
            asyncio.get_event_loop().create_task(self._connect_handler())

    async def stop(self):
        """Stop and clean up the ASCII listener."""
        self._state = STATE_STOPPED
        await self._disconnect_handler()

    async def _open_connection(self) -> bool:
        """Connect and login to HomeSeer ASCII at host:port."""
        # Attempt to connect to the ASCII connection at host:port
        try:
            connection = await asyncio.open_connection(self._host, self._port)
        except OSError as ex:
            _LOGGER.error(
                f"Error opening connection to HomeSeer ASCII at {self._host}:{self._port}: {ex}"
            )
            return False
        self._reader = connection[0]
        self._writer = connection[1]
        _LOGGER.info(
            f"Successful connection to HomeSeer ASCII at {self._host}:{self._port}"
        )

        # Create auth message and write it to the ASCII connection
        auth = "au,{},{}\r\n".format(self._username, self._password).encode()
        _LOGGER.debug(
            f"Logging in to HomeSeer ASCII connection at {self._host}:{self._port}"
        )
        self._writer.write(auth)
        await self._writer.drain()

        # Read response to auth message from the ASCII connection; expecting "ok"
        msg = await self._reader.readline()
        if msg.decode().strip() == "ok":
            _LOGGER.debug(
                f"Successful login to HomeSeer ASCII at {self._host}:{self._port}"
            )
        else:
            _LOGGER.error(
                f"Failed to login to HomeSeer ASCII at {self._host}:{self._port}: {msg.decode().strip()}"
            )
            return False

        # We are connected and logged in, set the ping flag and set state to connected
        self._ping_flag = True
        self._state = STATE_CONNECTED

        # If a connect callback has been provided, call it
        if self._async_connect_callback is not None:
            await self._async_connect_callback()

        return True

    async def _listen(self):
        """Listen for ASCII messages."""
        try:
            while True:
                msg = await self._reader.readline()
                _LOGGER.debug(
                    f"ASCII message received from {self._host}:{self._port}: {msg}"
                )
                if msg == b"":
                    raise HomeSeerASCIIConnectionError
                else:
                    await self._handle_message(msg.decode())

        except HomeSeerASCIIConnectionError:
            _LOGGER.warning(f"ASCII connection to {self._host}:{self._port} closed")
            await self._disconnect_handler()

        except Exception as ex:
            _LOGGER.error(
                f"ASCII listener error from connection to {self._host}:{self._port}: {ex}"
            )
            await self._disconnect_handler()

    async def _handle_message(self, raw):
        """Handle received messages from the ASCII connection."""
        # Raw msg format is Type,Data; break the msg into its separate parts
        msg = raw.split(",")
        # Telnet connection is active so set the ping flag to reset the ping timer
        self._ping_flag = True
        # We only care about DC messages
        if msg[0] == "DC":
            # "DC" is a "Device Change" message with format "DC,ref,newval,oldval"
            if self._async_message_callback is not None:
                # Call the callback with (ref) to signal that the device has changed
                await self._async_message_callback(msg[1])
        else:
            _LOGGER.debug(
                f"Unhandled ASCII message type received from {self._host}:{self._port}: {msg[0].strip()}"
            )

    async def _ping(self):
        """Pings the ASCII connection to maintain connection."""
        try:
            while True:
                if self._ping_flag:
                    _LOGGER.debug(
                        f"Resetting ASCII ping timer for {self._host}:{self._port} to {PING_TIMER} seconds"
                    )
                    self._ping_flag = False
                else:
                    _LOGGER.debug(
                        f"Pinging ASCII connection at {self._host}:{self._port}"
                    )
                    self._writer.write("vr\r\n".encode())
                    await self._writer.drain()
                await asyncio.sleep(PING_TIMER)
        except asyncio.CancelledError:
            _LOGGER.debug(
                f"Cancelling ping task for ASCII connection at {self._host}:{self._host}"
            )
            raise

    async def _connect_handler(self):
        """Called to attempt connect/reconnect after a delay."""
        if self.state != STATE_STOPPED:
            _LOGGER.info(
                f"Attempting to connect ASCII listener to {self._host}:{self._port} in {RECONNECT_TIMER} seconds"
            )
            await asyncio.sleep(RECONNECT_TIMER)
            await self.start()

    async def _disconnect_handler(self):
        """Called after a disconnection or error from the ASCII listener."""
        if self._ping_task is not None:
            self._ping_task.cancel()

        _LOGGER.debug(f"Closing ASCII listener at {self._host}:{self._port}")
        if self._writer is not None:
            self._writer.close()

        if self._async_disconnect_callback is not None:
            await self._async_disconnect_callback()

        await self._connect_handler()



from .const import *
from .devices import (
    HomeSeerStatusDevice,
    HomeSeerSwitchableDevice,
    HomeSeerDimmableDevice,
    HomeSeerLockableDevice,
)
from .helpers import *
from .homeseer import HomeSeer



"""Representation of API data for HomeSeer events as Python objects."""

from typing import Callable


class HomeSeerEvent:
    """Representation of a HomeSeer event."""

    def __init__(self, raw_data: dict, request: Callable) -> None:
        self._raw_data = raw_data
        self._request = request

    @property
    def group(self) -> str:
        """Return the group the event belongs to."""
        return self._raw_data["Group"]

    @property
    def name(self) -> str:
        """Return the name of the event."""
        return self._raw_data["Name"]

    async def run(self) -> None:
        """Run the event."""
        json = {"action": "runevent", "group": self.group, "name": self.name}
        await self._request("post", json=json)



"""Constants used in libhomeseer."""

DEFAULT_ASCII_PORT = 11000
DEFAULT_HTTP_PORT = 80
DEFAULT_PASSWORD = "default"
DEFAULT_USERNAME = "default"

DEVICE_ZWAVE_BARRIER_OPERATOR = "Z-Wave Barrier Operator"
DEVICE_ZWAVE_BATTERY = "Z-Wave Battery"
DEVICE_ZWAVE_CENTRAL_SCENE = "Z-Wave Central Scene"
DEVICE_ZWAVE_DOOR_LOCK = "Z-Wave Door Lock"
DEVICE_ZWAVE_DOOR_LOCK_LOGGING = "Z-Wave Door Lock Logging"
DEVICE_ZWAVE_ELECTRIC_METER = "Z-Wave Electric Meter"
DEVICE_ZWAVE_FAN_STATE = "Z-Wave Fan State"
DEVICE_ZWAVE_LUMINANCE = "Z-Wave Luminance"
DEVICE_ZWAVE_OPERATING_STATE = "Z-Wave Operating State"
DEVICE_ZWAVE_RELATIVE_HUMIDITY = "Z-Wave Relative Humidity"
DEVICE_ZWAVE_SENSOR_BINARY = "Z-Wave Sensor Binary"
DEVICE_ZWAVE_SENSOR_MULTILEVEL = "Z-Wave Sensor Multilevel"
DEVICE_ZWAVE_SWITCH = "Z-Wave Switch"
DEVICE_ZWAVE_SWITCH_BINARY = "Z-Wave Switch Binary"
DEVICE_ZWAVE_SWITCH_MULTILEVEL = "Z-Wave Switch Multilevel"
DEVICE_ZWAVE_TEMPERATURE = "Z-Wave Temperature"
DEVICE_ZWAVE_WATTS = "Z-Wave Watts"

RELATIONSHIP_ROOT = 2
RELATIONSHIP_STANDALONE = 3
RELATIONSHIP_CHILD = 4



"""Representations of API data for HomeSeer devices as Python objects."""

import logging
from typing import Callable, Optional, Union

from .const import RELATIONSHIP_CHILD, RELATIONSHIP_ROOT, RELATIONSHIP_STANDALONE

CONTROL_USE_ON = 1
CONTROL_USE_OFF = 2
CONTROL_USE_DIM = 3
CONTROL_USE_LOCK = 18
CONTROL_USE_UNLOCK = 19
CONTROL_USE_FAN = 23
CONTROL_LABEL_LOCK = "Lock"
CONTROL_LABEL_UNLOCK = "Unlock"

SUPPORT_STATUS = 0
SUPPORT_ON = 1
SUPPORT_OFF = 2
SUPPORT_LOCK = 4
SUPPORT_UNLOCK = 8
SUPPORT_DIM = 16
SUPPORT_FAN = 32

_LOGGER = logging.getLogger(__name__)


class HomeSeerStatusDevice:
    """
    Representation of a HomeSeer device with no controls (i.e. status only).
    Base representation for all other HomeSeer device objects.
    """

    def __init__(self, raw_data: dict, request: Callable) -> None:
        self._raw_data = raw_data
        self._request = request
        self._update_callback = None
        self._suppress_update_callback = False

    @property
    def ref(self) -> int:
        """Return the HomeSeer device ref of the device."""
        return int(self._raw_data["ref"])

    @property
    def name(self) -> str:
        """Return the name of the device."""
        return self._raw_data["name"]

    @property
    def location(self) -> str:
        """Return the location parameter of the device."""
        return self._raw_data["location"]

    @property
    def location2(self) -> str:
        """Return the location2 parameter of the device."""
        return self._raw_data["location2"]

    @property
    def value(self) -> Union[int, float]:
        """Return the value of the device."""
        if "." in str(self._raw_data["value"]):
            return float(self._raw_data["value"])
        return int(self._raw_data["value"])

    @property
    def status(self) -> str:
        """Return the status of the device."""
        return self._raw_data["status"]

    @property
    def device_type_string(self) -> Optional[str]:
        """Return the device type string of the device, or None for no type string (e.g. virtual device)."""
        if self._raw_data["device_type_string"]:
            return self._raw_data["device_type_string"]
        return None

    @property
    def last_change(self) -> str:
        """Return the last change of the device."""
        return self._raw_data["last_change"]

    @property
    def relationship(self) -> int:
        """
        Return the relationship of the device.
        2 = Root device (other devices may be part of this physical device)
        3 = Standalone (this is the only device that represents this physical device)
        4 = Child (this device is part of a group of devices that represent the physical device)
        """
        relationship = int(self._raw_data["relationship"])
        if relationship == RELATIONSHIP_ROOT:
            return RELATIONSHIP_ROOT
        elif relationship == RELATIONSHIP_CHILD:
            return RELATIONSHIP_CHILD
        elif relationship == RELATIONSHIP_STANDALONE:
            return RELATIONSHIP_STANDALONE
        return relationship

    @property
    def associated_devices(self) -> list:
        """
        A list of device reference numbers that are associated with this device.
        If the device is a Root device, the list contains the device reference numbers of the child devices.
        If the device is a Child device, the list will contain one device reference number of the root device.
        """
        return self._raw_data["associated_devices"]

    @property
    def interface_name(self) -> Optional[str]:
        """
        Return the name of the interface providing the device, or None for no interface (e.g. virtual device).
        Note: this parameter is present in the JSON API data but undocumented.
        """
        if self._raw_data["interface_name"]:
            return self._raw_data["interface_name"]
        return None

    def register_update_callback(
        self, callback: Callable, suppress_on_connection: bool = False
    ) -> None:
        """
        Register an update callback for the device, called when the device is updated by update_data.
        Set suppress_on_connection to True to suppress the callback on listener connect and disconnect.
        """
        self._suppress_update_callback = suppress_on_connection
        self._update_callback = callback

    def update_data(self, new_data: dict = None, connection_flag: bool = False) -> None:
        """Retrieve and cache updated data for the device from the HomeSeer JSON API."""
        if new_data is not None:
            _LOGGER.debug(
                f"Updating data for {self.location2} {self.location} {self.name} ({self.ref})"
            )
            self._raw_data = new_data

        if connection_flag and self._suppress_update_callback:
            return

        if self._update_callback is not None:
            self._update_callback()


class HomeSeerSwitchableDevice(HomeSeerStatusDevice):
    """Representation of a HomeSeer device that has On and Off control pairs."""

    def __init__(
        self, raw_data: dict, request: Callable, on_value: int, off_value: int
    ) -> None:
        super().__init__(raw_data, request)
        self._on_value = on_value
        self._off_value = off_value

    @property
    def is_on(self) -> bool:
        """Return True if the device's current value is greater than its off value."""
        return self.value > self._off_value

    async def on(self) -> None:
        """Turn the device on."""
        params = {
            "request": "controldevicebyvalue",
            "ref": self.ref,
            "value": self._on_value,
        }

        await self._request("get", params=params)

    async def off(self) -> None:
        """Turn the device off."""
        params = {
            "request": "controldevicebyvalue",
            "ref": self.ref,
            "value": self._off_value,
        }

        await self._request("get", params=params)


class HomeSeerDimmableDevice(HomeSeerSwitchableDevice):
    """Representation of a HomeSeer device that has a Dim control pair."""

    @property
    def dim_percent(self) -> float:
        """Returns a number from 0 to 1 representing the current dim percentage."""
        return self.value / self._on_value

    async def dim(self, percent: int) -> None:
        """Dim the device on a scale from 0 to 100."""
        if percent < 0 or percent > 100:
            raise ValueError("Percent must be an integer from 0 to 100")

        value = int(self._on_value * (percent / 100))

        params = {"request": "controldevicebyvalue", "ref": self.ref, "value": value}

        await self._request("get", params=params)


class HomeSeerFanDevice(HomeSeerSwitchableDevice):
    """Representation of a HomeSeer device that has a Fan or DimFan control pair."""

    @property
    def speed_percent(self) -> float:
        """Returns a number from 0 to 1 representing the current speed percentage."""
        return self.value / self._on_value

    async def speed(self, percent: int) -> None:
        """Set the speed of the device on a scale from 0 to 100."""
        if percent < 0 or percent > 100:
            raise ValueError("Percent must be an integer from 0 to 100")

        value = int(self._on_value * (percent / 100))

        params = {"request": "controldevicebyvalue", "ref": self.ref, "value": value}

        await self._request("get", params=params)


class HomeSeerLockableDevice(HomeSeerStatusDevice):
    """Representation of a HomeSeer device that has Lock and Unlock control pairs."""

    def __init__(
        self, raw_data: dict, request: Callable, lock_value: int, unlock_value: int
    ) -> None:
        super().__init__(raw_data, request)
        self._lock_value = lock_value
        self._unlock_value = unlock_value

    @property
    def is_locked(self) -> bool:
        """Return True if the device is locked."""
        return self.value == self._lock_value

    async def lock(self) -> None:
        """Lock the device."""
        params = {
            "request": "controldevicebyvalue",
            "ref": self.ref,
            "value": self._lock_value,
        }

        await self._request("get", params=params)

    async def unlock(self) -> None:
        """Unlock the device."""
        params = {
            "request": "controldevicebyvalue",
            "ref": self.ref,
            "value": self._unlock_value,
        }

        await self._request("get", params=params)


def get_device(
    raw_data: dict, control_data: dict, request: Callable
) -> Optional[
    Union[
        HomeSeerDimmableDevice,
        HomeSeerFanDevice,
        HomeSeerLockableDevice,
        HomeSeerStatusDevice,
        HomeSeerSwitchableDevice,
    ]
]:
    """
    Parses control_data to return an appropriate device object
    based on the control pairs detected for the device.
    On/Off = HomeSeerSwitchableDevice
    On/Off/Dim = HomeSeerDimmableDevice
    On/Off/Fan = HomeSeerFanDevice
    Lock/Unlock = HomeSeerLockableDevice
    other = HomeSeerStatusDevice
    """
    on_value = None
    off_value = None
    lock_value = None
    unlock_value = None
    control_pairs = None
    supported_features = SUPPORT_STATUS
    for item in control_data:
        if item["ref"] == raw_data["ref"]:
            control_pairs = item["ControlPairs"]
            for pair in control_pairs:
                control_use = pair["ControlUse"]
                control_label = pair["Label"]
                if control_use == CONTROL_USE_ON:
                    on_value = pair["ControlValue"]
                    supported_features |= SUPPORT_ON
                elif control_use == CONTROL_USE_OFF:
                    off_value = pair["ControlValue"]
                    supported_features |= SUPPORT_OFF
                elif (
                    control_use == CONTROL_USE_LOCK
                    or control_label == CONTROL_LABEL_LOCK
                ):
                    lock_value = pair["ControlValue"]
                    supported_features |= SUPPORT_LOCK
                elif (
                    control_use == CONTROL_USE_UNLOCK
                    or control_label == CONTROL_LABEL_UNLOCK
                ):
                    unlock_value = pair["ControlValue"]
                    supported_features |= SUPPORT_UNLOCK
                elif control_use == CONTROL_USE_DIM:
                    supported_features |= SUPPORT_DIM
                elif control_use == CONTROL_USE_FAN:
                    supported_features |= SUPPORT_FAN
            break

    if supported_features == SUPPORT_ON | SUPPORT_OFF:
        return HomeSeerSwitchableDevice(
            raw_data, request, on_value=on_value, off_value=off_value
        )

    elif supported_features == SUPPORT_ON | SUPPORT_OFF | SUPPORT_DIM:
        return HomeSeerDimmableDevice(
            raw_data, request, on_value=on_value, off_value=off_value
        )

    elif supported_features == SUPPORT_ON | SUPPORT_OFF | SUPPORT_FAN:
        return HomeSeerFanDevice(
            raw_data, request, on_value=on_value, off_value=off_value
        )

    elif supported_features == SUPPORT_LOCK | SUPPORT_UNLOCK:
        return HomeSeerLockableDevice(
            raw_data, request, lock_value=lock_value, unlock_value=unlock_value
        )

    else:
        _LOGGER.debug(
            f"Failed to automatically detect device Control Pairs for device ref {raw_data['ref']}; "
            f"creating a status-only device. "
            f"If this device has controls, open an issue on the libhomeseer repo "
            f"with the following information to request support for this device: "
            f"RAW: ({raw_data}) "
            f"CONTROL: ({control_pairs})."
        )
        return HomeSeerStatusDevice(raw_data, request)



import setuptools

long_description = open('README.md').read()

setuptools.setup(
    name="libhomeseer",
    version="1.3.0",
    author="Mark Coombes",
    author_email="mark@markcoombes.ca",
    description="Python3 async library for interacting with HomeSeer HS3 and HS4 via JSON and ASCII",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/marthoc/libhomeseer",
    packages=['libhomeseer'],
    install_requires=['asyncio', 'aiohttp'],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
)
